﻿/*
Maxscript:    [PS3] Soul Calibur III, Mesh Importer
Author:        mariokart64n
Date:           April 17 2020

-Mesh Import Issues
-Skeleton Import Issues
-Texture Unpacking Not Written yet
.... lots of todo

*/

clearlistener()

struct vmp_header (
	filetype = 0,
	unk001 = 0,
	unk002 = 0,
	unk003 = 0,
	unk004 = 0,
	table1_count = 0,
	bone_count = 0,
	unk005 = 0,
	unk006 = 0, -- padding?
	unk007 = 0, -- padding?
	unk008 = 0, -- padding?
	unk009 = 0, -- padding?
	-- addresses
	bone_addr = 0,
	tex_file_addr = 0,
	table1a_addr = 0,
	table5_addr = 0,
	table4_addr = 0,
	table7_addr = 0, -- jumps to a few ints and a string, then follows the string table (minus 64 from each byte in string table)
	unk016 = 0, -- always 0?
	table1_addr = 0,
	table2_addr = 0,
	table3_addr = 0,
	table6_addr = 0,
	fn read_header &f = (
		filetype = readlong f #unsigned
		if filetype == 0x0B1C0E09 then (
			unk001 = readbyte f #unsigned
			unk002 = readbyte f #unsigned
			unk003 = readbyte f #unsigned
			unk004 = readbyte f #unsigned
			table1_count = readshort f #unsigned
			bone_count = readshort f #unsigned
			unk005 = readlong f #unsigned
			unk006 = readlong f #unsigned
			unk007 = readlong f #unsigned
			unk008 = readlong f #unsigned
			unk009 = readlong f #unsigned
			bone_addr = readlong f #unsigned
			tex_file_addr = readlong f #unsigned
			table1a_addr = readlong f #unsigned
			table5_addr = readlong f #unsigned
			table4_addr = readlong f #unsigned
			table7_addr = readlong f #unsigned
			unk016 = readlong f #unsigned
			table1_addr = readlong f #unsigned
			table2_addr = readlong f #unsigned
			table3_addr = readlong f #unsigned
			table6_addr = readlong f #unsigned
			true
			)
		else (
			messagebox ("Invalid File: " + (filetype as string))
			false
			)
		)
	)
struct vmp_table1a ( -- mesh bone palette
	type = 0,	-- 0 = Static Mesh | 1 = Skinned Mesh
	count = 0,
	unk023 = 0,
	unk024 = 0,
	indices = #(),
	unk025 = 0,
	addrs = #(),
	fn read_table1a &f = (
		local i = 1
		type = readbyte f #unsigned
		count = readbyte f #unsigned
		unk023 = readbyte f #unsigned
		unk024 = readbyte f #unsigned
		
		case type of (
			0: () -- nothing?
			1: (
				if unk024 > 0 do indices[unk024] = 0
				for i = 1 to unk024 do (
					indices[i] = readbyte f #unsigned
					)
				fseek f (32 - unk024) #seek_cur
				)
			2: (
				readlong f #unsigned
				readfloat f
				readfloat f
				)
			3: ( -- addresses pointing to the geometry and a float table
				readfloat f
				readlong f #unsigned
				readlong f #unsigned
				readlong f #unsigned
				)
			4: (
				fseek f 0x9C #seek_cur
				)
			default: (
				format "Error Invalid Type:\t[%]\t@\t0x%\n" \
					type (bit.intAsHex (((ftell f) - 4) as integer))
				)
			)
		unk025 = readlong f #unsigned
		if count > 0 do addrs[count] = 0
		for i = 1 to count do (
			addrs[i] = readlong f #unsigned
			)
		)
	)
struct vmp_table1b (
	pos = 0,
	unk025 = 0,
	unk026 = 0,
	unk027 = 0,
	unk028 = 0,
	addr = 0,
	fn read_table1b &f = (
		pos = ftell f
		unk025 = readbyte f #unsigned
		unk026 = readbyte f #unsigned
		unk027 = readbyte f #unsigned
		unk028 = readbyte f #unsigned
		addr = readlong f #unsigned
		)
	)
struct vmp_table1c_chunk (
	pos = 0,
	index = 0,
	type = 0,
	size = 0,
	unk072 = 0,
	unk073 = 0,
	fn read_table1c_chunk &f = (
		pos = ftell f
		type = readbyte f #unsigned
		size = readbyte f #unsigned
		unk072 = readbyte f #unsigned
		unk073 = readbyte f #unsigned
		)
	)
struct vmp_table1c_cmd01 (
	header = vmp_table1c_chunk()
	)
struct vmp_table1c_cmd02 (
	header = vmp_table1c_chunk(),
	unk083 = 0,
	unk084 = 0,
	unk085 = 0,
	unk086 = 0,
	unk087 = 0,
	fn read_table1c_cmd02 &f = (
		unk083 = readlong f #unsigned
		unk084 = readlong f #unsigned
		unk085 = readlong f #unsigned
		unk086 = readlong f #unsigned
		unk087 = readlong f #unsigned
		)
	)
struct vmp_table1c_cmd03 (
	header = vmp_table1c_chunk(),
	unk088 = 0, -- texture addrress
	unk089 = 0, -- texture addrress
	unk090 = 0,
	unk091 = 0,
	fn read_table1c_cmd03 &f = (
		unk088 = readlong f #unsigned
		unk089 = readlong f #unsigned
		unk090 = readfloat f
		unk091 = readfloat f
		)
	)
struct vmp_table1c_cmd05 (
	header = vmp_table1c_chunk(),
	table1d_addr = 0, -- address to mesh table
	fn read_table1c_cmd05 &f = (
		table1d_addr = readlong f #unsigned
		)
	)
struct vmp_table1c_cmd11 (
	header = vmp_table1c_chunk(),
	unk079 = 0,
	unk080 = 0,
	unk081 = 0,
	unk082 = 0,
	fn read_table1c_cmd11 &f = (
		unk079 = readbyte f #unsigned
		unk080 = readbyte f #unsigned
		unk081 = readbyte f #unsigned
		unk082 = readbyte f #unsigned
		)
	)
struct vmp_table1c (
	pos = 0,
	cmd01_info = #(),
	cmd02_info = #(),
	cmd03_info = #(),
	cmd05_info = #(),
	cmd11_info = #(),
	fn read_table1c &f = (
		local cont = true, i = 0, h = vmp_table1c_chunk()
		pos = ftell f
		do (
			h.read_table1c_chunk(f)
			case h.type of (
				0x00: ( -- exit loop
					fseek f (h.pos + h.size) #seek_set
					cont = false
					exit
					)
				0x01: (
					append cmd01_info (vmp_table1c_cmd11 header:h)
					)
				0x02: (
					append cmd02_info (vmp_table1c_cmd02 header:h)
					cmd02_info[cmd02_info.count].read_table1c_cmd02(f)
					)
				0x03: (
					append cmd03_info (vmp_table1c_cmd03 header:h)
					cmd03_info[cmd03_info.count].read_table1c_cmd03(f)
					)
				0x05: (
					append cmd05_info (vmp_table1c_cmd05 header:h)
					cmd05_info[cmd05_info.count].read_table1c_cmd05(f)
					)
				0x0B: (
					i += 1
					h.index = i
					append cmd11_info (vmp_table1c_cmd11 header:h)
					cmd11_info[cmd11_info.count].read_table1c_cmd11(f)
					)
				default: (
					format "invalid mesh entry:\t%\t@\t0x%\n" \
						h.type (bit.intAsHex ((h.pos) as integer))
					)
				)
			fseek f (h.pos + h.size) #seek_set
			h = vmp_table1c_chunk index:i
			) while cont
		)
	)
struct vmp_table1d_chunk (
	pos = 0,
	index = 0,
	unk094 = 0,
	unk095 = 0,
	unk096 = 0,
	unk097 = 0, -- type?
	fn read_table1d_chunk &f = (
		pos = ftell f
		unk094 = readbyte f #unsigned
		unk095 = readbyte f #unsigned
		unk096 = readbyte f #unsigned
		unk097 = readbyte f #unsigned
		)
	)
struct vmp_table1d_type16 (
	header = vmp_table1d_chunk()
	)
struct vmp_table1d_type48 (
	header = vmp_table1d_chunk(),
	addr = 0,
	fn read_table1d_type48 &f= (
		addr = readlong f #unsigned
		)
	)
struct vmp_table1d_type96 (
	header = vmp_table1d_chunk()
	)
struct vmp_table1d (
	pos = 0,
	table1d_type16 = #(),
	table1d_type48 = #(),
	table1d_type96 = #(),
	fn read_table1d &f = (
		local h = (vmp_table1d_chunk index:1), cont = true, i = 1
		pos = ftell f
		do (
			h.read_table1d_chunk(f)
			case h.unk097 of (
				0x10: (
					append table1d_type16 (vmp_table1d_type16 header:h)
					)
				0x30: (
					append table1d_type48 (vmp_table1d_type48 header:h)
					table1d_type48[table1d_type48.count].read_table1d_type48(f)
					)
				0x60: (
					append table1d_type96 (vmp_table1d_type96 header:h)
					cont = false
					)
				default: (
					format "Invalid Mesh Type: %\t@\t0x%\n" \
						h.unk097 (bit.intAsHex (h.pos as integer))
					)
				)
			i += 1
			fseek f (mod (16-(mod (ftell f) 16)) 16) #seek_cur
			h = vmp_table1d_chunk index:i
			) while cont
		)
	)
struct vmp_table2 (
	unk061 = 0, -- always 1?
	unk062 = 0, -- count ?
	unk063 = 0, -- count ?
	fn read_table2 &f = (
		unk061 = readlong f #unsigned
		unk062 = readlong f #unsigned
		unk063 = readlong f #unsigned
		)
	)
struct vmp_table3 (
	unk064 = 0, -- always 1?
	fn read_table3 &f = (
		unk064 = readlong f #unsigned
		)
	)
struct vif_header (
	pos = 0,
	size = 0,
	unk100 = 0,	-- 1
	unk101 = 0,	-- 1
	unk102 = 0,	-- 0
	unk103 = 0,	-- 1
	unk104 = 0,
	unk105 = 0,
	unk106 = 0,
	unk107 = 0,
	unk108 = 0,
	unk109 = 0,
	unk110 = 0,
	unk111 = 0,
	unk112 = 0,
	unk113 = 0,
	unk114 = 0,
	fn read_vif_header &f = (
		pos = ftell f
		unk100 = readbyte f #unsigned
		unk101 = readbyte f #unsigned
		unk102 = readbyte f #unsigned
		unk103 = readbyte f #unsigned
		
		unk104 = readbyte f #unsigned
		unk105 = readbyte f #unsigned
		unk106 = readbyte f #unsigned
		unk107 = readbyte f #unsigned
		size = (((((bit.and unk107 0x0F) / 4) + 1) * 4) * unk106) - 8
		
		unk108 = readlong f #unsigned
		unk109 = readbyte f #unsigned
		unk110 = readbyte f #unsigned
		unk111 = readbyte f #unsigned
		unk112 = readbyte f #unsigned
		unk113 = readlong f #unsigned
		unk114 = readlong f #unsigned
		)
	)
-- struct vmp_table4_data1_entry (
-- 	type = 0,
-- 	addr = 0
-- 	)
-- struct vmp_table4_data1 (
-- 	type = 0,
-- 	addr1 = 0,
-- 	addr2 = 0,
-- 	count = 0,
-- 	data = #()
-- 	)
-- struct vmp_table4 (
-- 	unk053 = 0, -- count?
-- 	unk054 = 0, -- addr?
-- 	table4_data1 = #()
-- 	)
struct vmp_table4_header (
	type = 0,
	count = 0,
	unk055 = 0,
	unk056 = 0,
	fn read_table4_header &f = (
		type = readbyte f #unsigned
		if type > 0 do (
			count = readbyte f #unsigned
			unk055 = readbyte f #unsigned
			unk056 = readbyte f #unsigned
			)
		)
	)
struct vmp_table4_info01 (
	header = vmp_table4_header(),
	addr = 0
	)
struct vmp_table4_info02 (
	header = vmp_table4_header(),
	addr1 = 0,
	addr2 = 0
	)
struct vmp_table4_info03 (
	header = vmp_table4_header(),
	addr = 0
	)
struct vmp_table4_info04 (
	header = vmp_table4_header(),
	addr = 0
	)
struct vmp_table4_info05 (
	header = vmp_table4_header(),
	state = false
	)
struct vmp_table4_info06 (
	header = vmp_table4_header(),
	addr = 0
	)
struct vmp_table4_info07 (
	header = vmp_table4_header(),
	indices = #()
	)
struct vmp_table4_info08 (
	header = vmp_table4_header(),
	addr = 0
	)
struct vmp_table4_info09 (
	header = vmp_table4_header(),
	addr = 0
	)
struct vmp_table4_info10 (
	header = vmp_table4_header(),
	addr = 0
	)
struct vmp_table4_info11 (
	header = vmp_table4_header(),
	addr = 0
	)
struct vmp_table4_info12 (
	header = vmp_table4_header(),
	state = false
	)
struct vmp_table4_info13 (
	header = vmp_table4_header(),
	state = false
	)
struct vmp_table4_info14 (
	header = vmp_table4_header(),
	state = false
	)
struct vmp_table4_info16 (
	header = vmp_table4_header(),
	indices = #(),
	addr = 0
	)
struct vmp_table4_info19 (
	header = vmp_table4_header(),
	state = false
	)
struct vmp_table4 (
	info01 = vmp_table4_info01(),		-- end address of chunks
	info02 = vmp_table4_info02(),		-- address inside tex file pointing to texture address
	info03 = vmp_table4_info03(),		-- not documented
	info04 = vmp_table4_info04(),		-- addr?
	info05 = vmp_table4_info05(),		-- count?
	info06 = vmp_table4_info06(),		-- addr?
	info07 = vmp_table4_info07(),		-- bone palette?
	info08 = vmp_table4_info08(),		-- addr?
	info09 = vmp_table4_info09(),		-- addr?
	info10 = vmp_table4_info10(),		-- not documented
	info11 = vmp_table4_info11(),		-- addr?
	info12 = vmp_table4_info12(),		-- flag?
	info13 = vmp_table4_info13(),		-- flag?
	info14 = vmp_table4_info14(),		-- flag?
	info16 = vmp_table4_info16(),		-- bone palette?
	info19 = vmp_table4_info13(),		-- flag?
	fn read_table4_entry &f = (
		local h = vmp_table4_header(), i = 1, cont = true
		while info01.addr == 0 or ftell f < info01.addr do (
			h = vmp_table4_header()
			h.read_table4_header(f)
			case h.type of (
				0x00: ( -- exit
					cont = false
					exit
					)
				0x01: (
					info01.header = h
					info01.addr = readlong f #unsigned
					)
				0x02: (
					info02.header = h
					info02.addr1 = readlong f #unsigned
					info02.addr2 = readlong f #unsigned
					)
				0x03: (
					info03.header = h
					info03.addr = readlong f #unsigned
					)
				0x04: (
					info04.header = h
					info04.addr = readlong f #unsigned
					)
				0x05: (
					info05.header = h
					info05.state = true
					)
				0x06: (
					info06.header = h
					info06.addr = readlong f #unsigned
					)
				0x07: (
					info07.header = h
					if info07.header.count > 0 do info07.indices[info07.header.count] = 0
					for i = 1 to info07.header.count do (
						info07.indices[i] = readbyte f #unsigned
						)
					fseek f (0x20 - info07.header.count) #seek_cur
					)
				0x08: (
					info08.header = h
					info08.addr = readlong f #unsigned
					)
				0x09: (
					info09.header = h
					info09.addr = readlong f #unsigned
					)
				0x0A: (
					info10.header = h
					info10.addr = readlong f #unsigned
					)
				0x0B: (
					info11.header = h
					info11.addr = readlong f #unsigned
					)
				0x0C: (
					info12.header = h
					info12.state = true
					)
				0x0D: (
					info13.header = h
					info13.state = true
					)
				0x0E: (
					info14.header = h
					info14.state = true
					)
				0x10: (
					info16.header = h
					if info16.header.count > 0 do info16.indices[info16.header.count] = 0
					for i = 1 to info16.header.count do (
						info16.indices[i] = readbyte f #unsigned
						)
					fseek f (0x20 - info16.header.count) #seek_cur
					info16.addr = readlong f #unsigned
					)
				0x13: (
					info19.header = h
					info19.state = true
					)
				default: (
					format "failed to parse this dumbster:\t%\t@\t0x%\n" \
						h.type (bit.intAsHex ((ftell f - 4) as integer))
					if info01.addr > 0 then (
						fseek f info01.addr #seek_set
						exit
						)
					else (
						cont = false
						exit
						)
					)
				)
			)
		cont
		)
	)
struct vmp_table6 (
	count6a = 0,
	count6b = 0,
	count6c = 0,
	unk058 = 0,
	unk059 = 0,
	unk060 = 0,
	table6a_addr = 0,
	table6b_addr = 0,
	table6c_addr = 0,
	unk064 = 0,
	unk065 = 0,
	unk066 = 0,
	table6a = #(),
	table6b = #(),
	table6c = #(),
	fn read_table6 &f = (
		local i = 1
		count6a = readshort f #unsigned
		count6b = readshort f #unsigned
		count6c = readshort f #unsigned
		unk058 = readshort f #unsigned
		unk059 = readlong f #unsigned
		unk060 = readlong f #unsigned
		table6a_addr = readlong f #unsigned
		table6b_addr = readlong f #unsigned
		table6c_addr = readlong f #unsigned
		unk064 = readlong f #unsigned
		unk065 = readlong f #unsigned
		unk066 = readlong f #unsigned
		if table6a_addr > 0 do (
			fseek f table6a_addr #seek_set
			if count6a > 0 do table6a[count6a] = #(0.0, 0.0, 0.0, 0.0)
			for i = 1 to count6a do (table6a[i] = #(readfloat f, readfloat f, readfloat f, readfloat f))
			)
		if table6b_addr > 0 do (
			fseek f table6b_addr #seek_set
			if count6b > 0 do table6b[count6b] = #(0.0, 0.0, 0.0, 0.0)
			for i = 1 to count6b do (table6b[i] = #(readfloat f, readfloat f, readfloat f, readfloat f))
			)
		if table6c_addr > 0 do (
			fseek f table6c_addr #seek_set
			if count6c > 0 do table6c[count6c] = #(0.0, 0.0, 0.0, 0.0)
			for i = 1 to count6c do (table6c[i] = #(readfloat f, readfloat f, readfloat f, readfloat f))
			)
		)
	)
struct bone_table1 (	-- 64bytes
	unk034 = [0.0, 0.0, 0.0], -- rotation?
	unk035 = [0.0, 0.0, 0.0], -- position?
	unk036 = 0,
	unk037 = 0,
	unk038 = 0,
	name = "",
	name_addr = 0,
	unk043 = 0,
	unk044 = 0,
	unk045 = 0,
	unk046 = 0,
	unk047 = 0,
	unk048 = 0,
	unk049 = 0, -- Child
	unk050 = 0, -- Parent
	unk051 = 0, -- Index
	unk052 = 0, -- Type / Group
	fn read_bone_name &f = (
		local b = 0
		name = ""
		do (
			b = readbyte f #unsigned
			if b != 0x40 do (
				name += bit.intAsChar (b - 0x40)
				)
			) while b!=0x40 
		),
	fn enumBoneType num = (
		case num of (
			0x00:"Bone"
			0x01:"PropBone"
			0x02:"Fingers"
			0x03:"Mouth"
			0x04:"KneeArmor"
			0x05:"ThighArmor"
			0x06:"six"
			0x07:"ChestArmor"
			0x08:"AnkleArmor"
			0x09:"HipArmor"
			0x0A:"ten"
			0x0B:"Twist"
			0x0C:"Hip"
			0x0D:"thirteen"
			0x0E:"Null"
			default:"New"
			)
		),
	fn read_bone_table1 &f = (
		unk034 = [readfloat f, readfloat f, readfloat f] * (readfloat f) -- +?
		unk035 = [readfloat f, readfloat f, readfloat f] * (readfloat f)
		unk036 = readfloat f
		unk037 = readfloat f
		unk038 = readfloat f
		name_addr = readlong f #unsigned
		unk043 = readfloat f
		unk044 = readfloat f
		unk045 = readbyte f #unsigned
		unk046 = readbyte f #unsigned
		unk047 = readbyte f #unsigned
		unk048 = readbyte f #unsigned
		unk049 = readbyte f #unsigned
		unk050 = readbyte f #unsigned
		unk051 = readbyte f #unsigned
		unk052 = readbyte f #unsigned
		)
	)


struct vmp_file (
	header = vmp_header(),
	table1 = #(),
	table1a = #(),
	table1b = #(),
	table1c = #(),
	table1d = #(),
	table2 = vmp_table2(),
	table3 = vmp_table3(),
	bone_info = #(),
	table4 = #(),
-- 	table5 = vmp_table5(),
	table6 = vmp_table6(),
	fn read_file &f = (
		local i = 1, ii = 1, b = undefined, tb4
		local lowAddr = 0, highAddr = 0
		
		-- read header
		if (header.read_header(f)) then (
			
			-- read table 1
			if header.table1_addr > 0 do (
				if header.table1_count > 0 do table1[header.table1_count] = 0
				for i = 1 to header.table1_count do (
					table1[i] = readlong f #unsigned
					)
				
				-- read table 1a
				if header.table1_count > 0 do table1a[header.table1_count] = vmp_table1a()
				for i = 1 to header.table1_count do (
					table1a[i] = vmp_table1a()
					if table1[i] > 0 do (
						fseek f table1[i] #seek_set
						table1a[i].read_table1a(f)
						)
					)
				
				-- read table 1b
				if table1a.count > 0 do (
					lowAddr = 0
					highAddr = 0
					for i = 1 to header.table1_count do (
						for ii = 1 to table1a[i].addrs.count do (
							if table1a[i].addrs[ii] + 8 > highAddr do highAddr = table1a[i].addrs[ii] + 8
							)
						)
					lowAddr = highAddr
					for i = 1 to header.table1_count do (
						for ii = 1 to table1a[i].addrs.count do (
							if table1a[i].addrs[ii] < lowAddr do lowAddr = table1a[i].addrs[ii]
							)
						)
					ii = (highAddr - lowAddr) / 8
					if ii > 0 do table1b[ii] = vmp_table1b()
					fseek f lowAddr #seek_set
					for i = 1 to ii do (
						table1b[i] = vmp_table1b()
						table1b[i].read_table1b(f)
						)
					)
				
				-- read table1c
				if table1b.count > 0 do (
					lowAddr = 0
					highAddr = 0
					
					for i = 1 to table1b.count do (
						if table1b[i].addr + 4 > highAddr do highAddr = table1b[i].addr + 4
						)
					
					lowAddr = highAddr
					
					for i = 1 to table1b.count do (
						if table1b[i].addr < lowAddr do lowAddr = table1b[i].addr
						)
					
					
					--format "table1c From:\t0x%\n" (bit.intAsHex (lowAddr as integer))
					--format "table1c Tooo:\t0x%\n" (bit.intAsHex (highAddr as integer))
					fseek f lowAddr #seek_set
					while ftell f < highAddr do (
						append table1c (vmp_table1c())
						table1c[table1c.count].read_table1c(f)
						-- after each chunk, its padded with 0's?
						fseek f (mod (16-(mod (ftell f) 16)) 16) #seek_cur
						)
					)
				
				-- read table1d
				if table1c.count > 0 do (
					lowAddr = 0
					highAddr = 0
					
					for i = 1 to table1c.count do (
						for ii = 1 to table1c[i].cmd05_info.count do (
							if table1c[i].cmd05_info[ii].table1d_addr + 16 > highAddr do highAddr = table1c[i].cmd05_info[ii].table1d_addr + 16
							)
						)
					
					lowAddr = highAddr
					for i = 1 to table1c.count do (
						for ii = 1 to table1c[i].cmd05_info.count do (
							if table1c[i].cmd05_info[ii].table1d_addr < lowAddr do lowAddr = table1c[i].cmd05_info[ii].table1d_addr
							)
						)
					
					--format "table1d From:\t0x%\n" (bit.intAsHex (lowAddr as integer))
					--format "table1d Tooo:\t0x%\n" (bit.intAsHex (highAddr as integer))
					fseek f lowAddr #seek_set
					while ftell f < highAddr do (
						append table1d (vmp_table1d())
						table1d[table1d.count].read_table1d(f)
						)
					)
				)
			
			-- read table2
			if header.table2_addr > 0 do (
				fseek f header.table2_addr #seek_set
				table2.read_table2(f)
				)
			
			-- read table3
			if header.table3_addr > 0 do (
				fseek f header.table3_addr #seek_set
				table3.read_table3(f)
				)
			
			-- read table4
			if header.table4_addr > 0 do (
				fseek f header.table4_addr #seek_set
				format "table4:\t0x%\n" (bit.intAsHex ((ftell f) as integer))
				tb4 = vmp_table4()
				while (tb4.read_table4_entry(f)) and ftell f < header.table5_addr do (
					append table4 tb4
					tb4 = vmp_table4()
					)
				)
			
			-- read table 5
-- 			if header.table5_addr > 0 and table2.count > 0 do (
-- 				fseek f header.table5_addr #seek_set
-- 				table5.unk053 = readlong f #unsigned
-- 				table5.unk054 = readlong f #unsigned
-- 				if header.table1_count > 0 do table5.table5_data1[header.table1_count] = vmp_table5_data1()
-- 				for i = 1 to header.table1_count do (
-- 					table5.table5_data1[i] = vmp_table5_data1()
-- 					table5.table5_data1[i].type = readlong f #unsigned
-- 					table5.table5_data1[i].addr1 = readlong f #unsigned
-- 					case table2[i].type of (
-- 						0: (
-- 							table5.table5_data1[i].addr2 = readlong f #unsigned
-- 							)
-- 						)
-- 					table5.table5_data1[i].count = (table5.table5_data1[i].addr1 - (ftell f)) / 8
-- 					
-- 					if table5.table5_data1[i].type != 1 do (
-- 						format "Unknown Type:\t%\t@\t0x%\n" \
-- 							table5.table5_data1[i].type \
-- 							(bit.intAsHex ((ftell f - 8) as integer))
-- 						)
-- 					
-- 					if table5.table5_data1[i].count > 0 do (
-- 						table5.table5_data1[i].data[table5.table5_data1[i].count] = vmp_table5_data1_entry()
-- 						)
-- 					
-- 					for ii = 1 to table5.table5_data1[i].count do (
-- 						table5.table5_data1[i].data[ii] = (
-- 							vmp_table5_data1_entry \
-- 								type:(readlong f #unsigned) \
-- 								addr:(readlong f #unsigned)
-- 							)
-- 						)
-- 					)
-- 				)
			
			
			-- read table6
			if header.table6_addr > 0 do (
				fseek f header.table6_addr #seek_set
				table6.read_table6(f)
				)
			
			-- read bone table
			if header.bone_addr > 0 do (
				fseek f header.bone_addr #seek_set
				if header.bone_count > 0 do bone_info[header.bone_count] = bone_table1()
				for i = 1 to header.bone_count do (
					bone_info[i] = bone_table1()
					bone_info[i].read_bone_table1(f)
					)
				
				-- read string table
				for i = 1 to header.bone_count do (
					if bone_info[i].name_addr > 0 do (
						fseek f bone_info[i].name_addr #seek_set
						bone_info[i].read_bone_name(f)
						)
					)
				)
			format "Location:\t0x%\n" (bit.intAsHex ((ftell f) as integer))
			--print table2
			--print "------------------"
			--print table2_sub1
			--print table4.table4_data1
			--print bone_info
			--print table6
			
			
			) else (format "File Exited\n")
		),
	fn generateFaces &faceArray num_verts offset:0 = (
		local i = 1, fcnt = 0, bf = 1
		
		for i = 3 to num_verts do (
			fcnt+=1
			fa=fcnt
			fb=fcnt+1
			fc=fcnt+2
			if bf==1 do append faceArray ([fa,fb,fc] + offset)
			if bf==-1 do append faceArray ([ fc,fb,fa] + offset)
			bf *= -1
			)
		),
	fn build &f = (
		local i = 1, ii = 1, b = 1, vif = vif_header()
		local vertArray = #(), normArray = #(), tvertArray = #(), faceArray = #(), chk = 0, v = 1
		local c1 = 0, c2 = 0, c3 = 0, c4 = 0, cpos = 0, mscale = 1.0 / 0.0254, num = "", unused_count = 0
		local vert_offset = 0, num_verts = 0, vert = [0, 0, 0], vert2 = [0, 0, 0], tvert = [0, 0, 0], norm = [0, 0, 0]
		local idstr = "", weightArray = #(), boneidArray = #(), wt = 1.0, bi = 1, idstr2 = ""
		local matArray = #()
		delete $*
		
		
		-- Construct Skeleton
		local boneArray = #(), bone_node = undefined, root_bone = undefined
		
		root_bone = BoneSys.createBone [0.0, 0.0, 0.0] [0.0, 0.0, 1.0] [1, 0, 0]
		root_bone.name = "000 ROOT"
		for i = 1 to bone_info.count do (
			bone_node = BoneSys.createBone (bone_info[i].unk035 * mscale) ((bone_info[i].unk035 * mscale) + [1.0, 0.0, 0.0]) [1, 0, 0]
			
			num = "000" + (i as string)
			bone_node.name = substring num (num.count - 2) 3
			
			if bone_info[i].name != "" then (
				bone_node.name += " " + bone_info[i].name
				)
			else (
				unused_count+=1
				bone_node.name += " UNUSED_" + (unused_count as string)
				)
			bone_node.name +=  " [" + (bone_info[i].unk052 as string) + "]"
			bone_node.transform = (
				(eulerAngles \
					(bone_info[i].unk036 * 360.0) \
					(bone_info[i].unk037 * 360.0) \
					(bone_info[i].unk038 * 360.0)
					) as matrix3
				)
			case bone_info[i].unk052 of (
				0x03: bone_node.transform *= rotateYPRMatrix 90 0 90
				--0x07: bone_node.transform *= rotateYPRMatrix 0 0 -10
				)
			
			
			bone_node.position = bone_info[i].unk035 * mscale
			append boneArray bone_node
			)
		for i = 1 to bone_info.count do (
			if bone_info[i].unk050 < boneArray.count then (
				boneArray[i].transform *= boneArray[bone_info[i].unk050 + 1].transform
				boneArray[i].parent = boneArray[bone_info[i].unk050 + 1]
				boneArray[bone_info[i].unk050 + 1].length = distance boneArray[bone_info[i].unk050 + 1].position boneArray[i].position
				)
			else (
				--boneArray[i].transform *= rotateYPRMatrix 0 90 0
				boneArray[i].parent = root_bone
				)
			)
		
		-- Construct Mesh
		local mesh_object = 1, mesh_element_index = 0, mesh_element = 1
		local mesh_mat = 1, mesh_mat_index = 0, t1d_index = 1
		local t1c_index = 0, t1b_index = 0, tfm = matrix3 1, flipflop = true, lastflop = false
		local poly_strip = 1, poly_strip_isShifted = false, strip_type = 0, mat = undefined
		local msh = undefined, bm_chk = #(), face_increment = 0, face_buffer = [0, 0, 0]
		local table1c_visited = #()
		
		for mesh_object = 1 to table1a.count do (
			vertArray = #()
			tvertArray = #()
			normArray = #()
			faceArray = #()
			weightArray = #()
			boneidArray = #()
			matArray = #()
			idstr = ""
			idstr2 = ""
			num_verts = 0
			face_increment = 0
			face_buffer = [0, 0, 0]
			for mesh_element = 1 to table1a[mesh_object].count do (
				t1b_index = 0
				for i = 1 to table1b.count do (
					if table1a[mesh_object].addrs[mesh_element] == table1b[i].pos do (
						t1b_index = i
						exit
						)
					)
				if t1b_index > 0 then (
					t1c_index = 0
					--print table4[t1b_index].info11
					for i = 1 to table1c.count do (
						if table1b[t1b_index].addr == table1c[i].pos do (
							t1c_index = i
							append table1c_visited table1c[i].pos
							exit
							)
						)
					if t1c_index > 0 then (
-- 						format "unk025:\t%\n" table1b[t1c_index].unk025
-- 						format "01:\t%\t02:\t%\t03:\t%\t04:\t%\t11:\t%\n" \
-- 							table1c[t1c_index].cmd01_info.count \
-- 							table1c[t1c_index].cmd02_info.count \
-- 							table1c[t1c_index].cmd03_info.count \
-- 							table1c[t1c_index].cmd05_info.count \
-- 							table1c[t1c_index].cmd11_info.count
-- 						
-- 						print table1c[t1c_index].cmd05_info
						
						for mesh_mat = 1 to table1c[t1c_index].cmd02_info.count do (
							fseek f table1c[t1c_index].cmd02_info[mesh_mat].unk087 #seek_set
							/*
								for some reason sometimes the data is shift over 8bytes???
								for this reason I'm just gonna scan over to locate it
								this strange behavior has been seen all around the file structure
								might be a way to indicate a flag? seems dumb
								*/
							do (b = readlong f #unsigned) while b == 0
							fseek f -4 #seek_cur
							poly_strip_isShifted = false
							if ftell f != table1d[mesh_mat_index].table1d_type48[poly_strip].addr do (
								poly_strip_isShifted = true
								)
							
							flipflop = true
							lastflop = true
							idstr += " 0x" + (bit.intAsHex ((ftell f) as integer))
							-- read vif command
							vif.read_vif_header(f)
							

-- 							format "Object:\t%-%\tpoly_strip_isShifted:\t%\n" \
-- 								mesh_object mesh_element poly_strip_isShifted
							
-- 							format "Mesh %-%\t0x%\t%\t%\t%\t%\t%\t%\t%\t%\t%\t%\t%\t%\t%\t%\t%\t%\n" \
-- 								i ii (bit.intAsHex (table1d[i].table1d_type48[ii].addr as integer)) \
-- 								vif.unk100 vif.unk101 vif.unk102 vif.unk103 vif.unk104 vif.unk105 vif.unk106 \
-- 								vif.unk107 vif.unk108 (bit.and vif.unk108 0x000000FF) vif.unk109 vif.unk110 vif.unk111 vif.unk112 \
-- 								vif.unk113 vif.unk114
							
							-- some types appear to be identical?
							strip_type = vif.unk114
							if vif.unk114 == 1 do strip_type = 0
							if vif.unk114 == 15 do strip_type = 14
							
							vert = [0, 0, 0]
							vert2 = [0, 0, 0]
							tvert = [0, 0, 0]
							norm = [0, 0, 0]
							
							idstr2+=" " + vif.unk114 as string
							
							-- read strip
							case table1a[mesh_object].type of (
								0x00: (	-- Static Mesh
									-- Series
									b = (readbyte f #unsigned)
									readbyte f #unsigned
									readbyte f #unsigned
									readbyte f #unsigned
									for chk = 1 to b do (
										c1 = readbyte f #unsigned
										c2 = readbyte f #unsigned
										c3 = readbyte f #unsigned
										c4 = readbyte f #unsigned
										--format "%\t%\t%\t%\n" c1 c2 c3 c4
										cpos = ftell f
										if c2 == 0x80 then (
											case c1 of (
												0x01: ( -- texture coord
													for v = 1 to c3 do (
														append tvertArray [readfloat f, readfloat f, 0]
														
														)
													)
												0x02: ( -- colour 
													for v = 1 to c3 do (readfloat f)
													)
												0x03: ( -- position
													
													for v = 1 to c3 do (
														
														--fseek f 4 #seek_cur
														--chk = readlong f #unsigned
														--fseek f -4 #seek_cur
														
														
														append vertArray ([readfloat f, readfloat f, readfloat f] * mscale)
														-- for reading a long: bit.shift (bit.and 4080 0x0000F000) -12
														readbyte f #unsigned
														
														face_increment += 1
														face_buffer = [face_buffer[2], face_buffer[3], face_increment]
														
														
														--flipflop = if (bit.get chk 30) then true else false
														
														
														if (bit.shift (bit.and (readbyte f #unsigned) 0xF0) -4) == 0 then (
															lastflop = flipflop
															if flipflop then (
																flipflop = false
																append faceArray [face_buffer[2], face_buffer[1], face_buffer[3]]
																)
															else (
																append faceArray face_buffer
																flipflop = true
																)
															append matArray mesh_mat
															)
														else (
															flipflop = lastflop
															
															)
														
														
														readbyte f #unsigned
														readbyte f #unsigned
														append weightArray #(1.0)
														append boneidArray #(table1a[mesh_object].unk024 + 1)
														)
													
													--generateFaces faceArray (c3 / 3) offset:num_verts
													
													num_verts += c3
													)
												0x04: ( -- normal
													for v = 1 to c3 do (append normArray [readfloat f, readfloat f, readfloat f])
													)
												default: (
													format "Unknown Vertex Component [%]:\t@\t0x%\n" \
														c1 (bit.intAsHex ((ftell f) as integer))
													)
												)
											)
										else (
											--lastflop = flipflop
											if lastflop then (
												
												lastflop = false
												)
											else (
												
												lastflop = true
												)
											flipflop = lastflop
											)
										fseek f (cpos + (((((bit.and c4 0x0F) / 4) + 1) * 4) * c3)) #seek_set
										)
									if tvertArray.count < vertArray.count do (
										for v = 1 to (vertArray.count - tvertArray.count) do (
											append tvertArray [0.0, 0.0, 0.0]
											)
										)
									
									)
								0x01: (	-- Skinned Mesh
									-- Interweave
									--format "Object %\n%\n" mesh_object (table1a[mesh_object].indices as string)
									
									b = 0
									--# read geometry data until null is reached
									while ftell f < (vif.pos + vif.size) do (
										--# read and store vertex components
										--chk = readlong f #unsigned
										--fseek f -4 #seek_cur
										--if chk != 0 and chk != 0x01000101 do (
										b+=1
										
										vert2 = [readfloat f, readfloat f, readfloat f] * mscale
										wt = readfloat f
										
										
										norm = [readfloat f, readfloat f, readfloat f]
										
										--# read unknown data
										bi = readbyte f #unsigned
										v = readbyte f #unsigned
										readshort f #unsigned
										
										--vert += (vert2 + bone_info[table1a[mesh_object].indices[(bit.shift bi -2) + 1] + 1].unk035) * wt
										
										tfm = matrix3 1
										tfm.row4 = vert2 / wt
										tfm*=boneArray[table1a[mesh_object].indices[(bit.shift bi -2) + 1] + 1].transform
										vert+= tfm.row4  * wt
										
										
										
										
-- 													if mesh_object == 2 do (

-- 														appendifunique bm_chk table1a[mesh_object].indices[(bit.shift bi -2) + 1]
-- 														)
										
										--append weightArray #(wt, 1.0 - wt)
										--append boneidArray #(table1a[mesh_object].unk024 + 1, bi + 1)
										
										
										
										--# read / skip depending on byte
										case v of (
											0x00: (
												--lastflop = flipflop
												if lastflop then (
													
													lastflop = false
													)
												else (
													
													lastflop = true
													)
												flipflop = lastflop
												)
											0x80: (
												append vertArray vert
												append normArray norm
												norm = [0,0,0]
												vert = [0,0,0]
												vert2 = [0,0,0]
												append tvertArray [readfloat f, readfloat f, 0]
												readfloat f -- always 1?
												
												face_increment += 1
												face_buffer = [face_buffer[2], face_buffer[3], face_increment]
												
												
	
												
												
												if readlong f #unsigned == 0 then (
													lastflop = flipflop
													if flipflop then (
														append faceArray [face_buffer[2], face_buffer[1], face_buffer[3]]
														flipflop = false
														)
													else (
														append faceArray face_buffer
														flipflop = true
														)
													append matArray mesh_mat
													)
												else (
													flipflop = lastflop
													
													)
												
												readlong f #unsigned
												readlong f #unsigned
												readlong f #unsigned
												readlong f #unsigned
												)
											default: (
												format "---------------------------------------------read error occurred:\t0x%\n" (bit.intAsHex ((ftell f) as integer))
												exit
												)
											)
											--)
										)-- while chk != 0 and chk != 0x01000101
			-- 							format "Actual Count is:\t%\n" b
									)
								default: (
									format "-------------------------------------------------------Unknown Vertex Type [%]:\t0x%\n\n" \
										vif.unk114 (bit.intAsHex ((ftell f) as integer))
									exit
									)
								)
							
							)
						for mesh_mat = 1 to table1c[t1c_index].cmd05_info.count do (
							mesh_mat_index = 0
							for t1d_index = 1 to table1d.count do (
								if table1c[t1c_index].cmd05_info[mesh_mat].table1d_addr == table1d[t1d_index].pos do (
									mesh_mat_index = t1d_index
									exit
									)
								)
							if mesh_mat_index > 0 then (
								for poly_strip = 1 to table1d[mesh_mat_index].table1d_type48.count do (
									fseek f table1d[mesh_mat_index].table1d_type48[poly_strip].addr #seek_set
									/*
										for some reason sometimes the data is shift over 8bytes???
										for this reason I'm just gonna scan over to locate it
										this strange behavior has been seen all around the file structure
										might be a way to indicate a flag? seems dumb
										*/
									do (b = readlong f #unsigned) while b == 0
									fseek f -4 #seek_cur
									poly_strip_isShifted = false
									if ftell f != table1d[mesh_mat_index].table1d_type48[poly_strip].addr do (
										poly_strip_isShifted = true
										)
									
									flipflop = true
									lastflop = true
									idstr += " 0x" + (bit.intAsHex ((ftell f) as integer))
									-- read vif command
									vif.read_vif_header(f)
									

		-- 							format "Object:\t%-%\tpoly_strip_isShifted:\t%\n" \
		-- 								mesh_object mesh_element poly_strip_isShifted
									
		-- 							format "Mesh %-%\t0x%\t%\t%\t%\t%\t%\t%\t%\t%\t%\t%\t%\t%\t%\t%\t%\t%\n" \
		-- 								i ii (bit.intAsHex (table1d[i].table1d_type48[ii].addr as integer)) \
		-- 								vif.unk100 vif.unk101 vif.unk102 vif.unk103 vif.unk104 vif.unk105 vif.unk106 \
		-- 								vif.unk107 vif.unk108 (bit.and vif.unk108 0x000000FF) vif.unk109 vif.unk110 vif.unk111 vif.unk112 \
		-- 								vif.unk113 vif.unk114
									
									-- some types appear to be identical?
									strip_type = vif.unk114
									if vif.unk114 == 1 do strip_type = 0
									if vif.unk114 == 15 do strip_type = 14
									
									vert = [0, 0, 0]
									vert2 = [0, 0, 0]
									tvert = [0, 0, 0]
									norm = [0, 0, 0]
									
									idstr2+=" " + vif.unk114 as string
									
									-- read strip
									case table1a[mesh_object].type of (
										0x00: (	-- Static Mesh
											-- Series
											b = (readbyte f #unsigned)
											readbyte f #unsigned
											readbyte f #unsigned
											readbyte f #unsigned
											for chk = 1 to b do (
												c1 = readbyte f #unsigned
												c2 = readbyte f #unsigned
												c3 = readbyte f #unsigned
												c4 = readbyte f #unsigned
												--format "%\t%\t%\t%\n" c1 c2 c3 c4
												cpos = ftell f
												if c2 == 0x80 then (
													case c1 of (
														0x01: ( -- texture coord
															for v = 1 to c3 do (
																append tvertArray [readfloat f, readfloat f, 0]
																
																)
															)
														0x02: ( -- colour 
															for v = 1 to c3 do (readfloat f)
															)
														0x03: ( -- position
															
															for v = 1 to c3 do (
																
																--fseek f 4 #seek_cur
																--chk = readlong f #unsigned
																--fseek f -4 #seek_cur
																
																
																append vertArray ([readfloat f, readfloat f, readfloat f] * mscale)
																-- for reading a long: bit.shift (bit.and 4080 0x0000F000) -12
																readbyte f #unsigned
																
																face_increment += 1
																face_buffer = [face_buffer[2], face_buffer[3], face_increment]
																
																
																--flipflop = if (bit.get chk 30) then true else false
																
																
																if (bit.shift (bit.and (readbyte f #unsigned) 0xF0) -4) == 0 then (
																	lastflop = flipflop
																	if flipflop then (
																		flipflop = false
																		append faceArray [face_buffer[2], face_buffer[1], face_buffer[3]]
																		)
																	else (
																		append faceArray face_buffer
																		flipflop = true
																		)
																	append matArray mesh_mat
																	)
																else (
																	flipflop = lastflop
																	
																	)
																
																
																readbyte f #unsigned
																readbyte f #unsigned
																append weightArray #(1.0)
																append boneidArray #(table1a[mesh_object].unk024 + 1)
																)
															
															--generateFaces faceArray (c3 / 3) offset:num_verts
															
															num_verts += c3
															)
														0x04: ( -- normal
															for v = 1 to c3 do (append normArray [readfloat f, readfloat f, readfloat f])
															)
														default: (
															format "Unknown Vertex Component [%]:\t@\t0x%\n" \
																c1 (bit.intAsHex ((ftell f) as integer))
															)
														)
													)
												else (
													--lastflop = flipflop
													if lastflop then (
														
														lastflop = false
														)
													else (
														
														lastflop = true
														)
													flipflop = lastflop
													)
												fseek f (cpos + (((((bit.and c4 0x0F) / 4) + 1) * 4) * c3)) #seek_set
												)
											if tvertArray.count < vertArray.count do (
												for v = 1 to (vertArray.count - tvertArray.count) do (
													append tvertArray [0.0, 0.0, 0.0]
													)
												)
											
											)
										0x01: (	-- Skinned Mesh
											-- Interweave
											--format "Object %\n%\n" mesh_object (table1a[mesh_object].indices as string)
											
											b = 0
											--# read geometry data until null is reached
											while ftell f < (vif.pos + vif.size) do (
												--# read and store vertex components
												--chk = readlong f #unsigned
												--fseek f -4 #seek_cur
												--if chk != 0 and chk != 0x01000101 do (
												b+=1
												
												vert2 = [readfloat f, readfloat f, readfloat f] * mscale
												wt = readfloat f
												
												
												norm = [readfloat f, readfloat f, readfloat f]
												
												--# read unknown data
												bi = readbyte f #unsigned
												v = readbyte f #unsigned
												readshort f #unsigned
												
												--vert += (vert2 + bone_info[table1a[mesh_object].indices[(bit.shift bi -2) + 1] + 1].unk035) * wt
												
												tfm = matrix3 1
												tfm.row4 = vert2 / wt
												tfm*=boneArray[table1a[mesh_object].indices[(bit.shift bi -2) + 1] + 1].transform
												vert+= tfm.row4  * wt
												
												
												
												
-- 													if mesh_object == 2 do (

-- 														appendifunique bm_chk table1a[mesh_object].indices[(bit.shift bi -2) + 1]
-- 														)
												
												--append weightArray #(wt, 1.0 - wt)
												--append boneidArray #(table1a[mesh_object].unk024 + 1, bi + 1)
												
												
												
												--# read / skip depending on byte
												case v of (
													0x00: (
														--lastflop = flipflop
														if lastflop then (
															
															lastflop = false
															)
														else (
															
															lastflop = true
															)
														flipflop = lastflop
														)
													0x80: (
														append vertArray vert
														append normArray norm
														norm = [0,0,0]
														vert = [0,0,0]
														vert2 = [0,0,0]
														append tvertArray [readfloat f, readfloat f, 0]
														readfloat f -- always 1?
														
														face_increment += 1
														face_buffer = [face_buffer[2], face_buffer[3], face_increment]
														
														
			
														
														
														if readlong f #unsigned == 0 then (
															lastflop = flipflop
															if flipflop then (
																append faceArray [face_buffer[2], face_buffer[1], face_buffer[3]]
																flipflop = false
																)
															else (
																append faceArray face_buffer
																flipflop = true
																)
															append matArray mesh_mat
															)
														else (
															flipflop = lastflop
															
															)
														
														readlong f #unsigned
														readlong f #unsigned
														readlong f #unsigned
														readlong f #unsigned
														)
													default: (
														format "---------------------------------------------read error occurred:\t0x%\n" (bit.intAsHex ((ftell f) as integer))
														exit
														)
													)
													--)
												)-- while chk != 0 and chk != 0x01000101
					-- 							format "Actual Count is:\t%\n" b
											)
										default: (
											format "-------------------------------------------------------Unknown Vertex Type [%]:\t0x%\n\n" \
												vif.unk114 (bit.intAsHex ((ftell f) as integer))
											exit
											)
										)
									)
								) else (format "\nError:\tUnlinked Strip\n")
							)
						
						) else (format "error t1c_index \n")
					) else (format "error table1b\n")
				)
			--generateFaces faceArray vertArray.count
			msh = mesh vertices:vertArray faces:faceArray tverts:tvertArray materialIDs:matArray
			mat = multimaterial numsubs:table1c[t1c_index].cmd05_info.count
			for i = 1 to mat.count do (mat[i].diffuse = random white black)
			msh.material = mat
			msh.wirecolor = random (color 0 0 0) (color 255 255 255)
			msh.backfacecull = on
			if tvertArray.count > 0 do (
				buildTVFaces msh
				for i = 1 to tvertArray.count do setTVert msh i tvertArray[i]
				for i = 1 to faceArray.count do setTVFace msh i faceArray[i]
				)
			
			num = "000" + (mesh_object as string)
			msh.name = "mesh " + (substring num (num.count - 2) 3) + (if table1a[mesh_object].type == 0 then " static" else " skinned") + idstr
			if table1a[mesh_object].unk024 < boneArray.count do (
				if table1a[mesh_object].type == 0 then (
					msh.transform = boneArray[table1a[mesh_object].unk024 + 1].transform
					msh.parent = boneArray[table1a[mesh_object].unk024 + 1]
					)
				else (
					msh.transform = matrix3 1
					msh.parent = boneArray[table1a[mesh_object].unk024 + 1]
					)
				)
			)

		--print bm_chk
-- 			print bone_info
-- 		print table1
-- 		print table1a
-- 		print table1b
-- 		print table1c
-- 		print table1d
-- 		print table4

		)
	)
fn main file = (
	local f = undefined, vmp = vmp_file()
	f = try(fopen file "rbS")catch(undefined)
	if f != undefined then (
		vmp.read_file(f)
		vmp.build(f)
		fclose f
		) else (messagebox "Failed to Open file")
	)
main(
	"G:\\_HACKS&MODS\\SoulCalibur\\SC3_Playstation\\07-0009-01.vmp"
-- 	"G:\\_HACKS&MODS\\SoulCalibur\\SC3_Playstation\\07-0372-01.vmp"
-- 	"G:\\_HACKS&MODS\\SoulCalibur\\SC3_Playstation\\05-0001-01.vmp"
-- 	"G:\\_HACKS&MODS\\SoulCalibur\\SC3_Playstation\\05-0015-01.vmp"
-- 	"G:\\_HACKS&MODS\\SoulCalibur\\SC3_Playstation\\07-0033-01.vmp"
-- 	"G:\\_HACKS&MODS\\SoulCalibur\\SC3_Playstation\\07-0374-01.vmp"
	)